from __future__ import annotations


ACCOUNT_LOGIN_MODE_VALUES: tuple[str, ...] = ("local", "remote")
ACCOUNT_LOGIN_TIMEOUT_DEFAULT = 240
ACCOUNT_LOGIN_TIMEOUT_MIN = 30
BOOLEAN_LITERAL_VALUES: tuple[str, ...] = ("true", "false")
NOTE_GET_COMMENT_SIZE_DEFAULT = 10
NOTE_GET_COMMENT_SIZE_MIN = 10
NOTE_GET_SUB_COMMENT_SIZE_DEFAULT = 5
NOTE_GET_SUB_COMMENT_SIZE_MIN = 5
NOTE_XSEC_SOURCE_VALUES: tuple[str, ...] = (
    "pc_feed",
    "pc_search",
    "pc_collect",
    "pc_like",
    "pc_user",
)
NOTE_SEARCH_SIZE_DEFAULT = 20
NOTE_SEARCH_SIZE_MIN = 20
PUBLISH_TARGET_VALUES: tuple[str, ...] = ("image", "video")
USER_XSEC_SOURCE_VALUES: tuple[str, ...] = (
    "pc_feed",
    "pc_search",
    "pc_collect",
    "pc_like",
    "pc_note",
)
USER_SEARCH_SIZE_DEFAULT = 20
USER_SEARCH_SIZE_MIN = 20